#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <stdlib.h>
#include "lcd.h"

int main(void)
{
    if (button_pressed(0))
    {
			main_pong();
    }
    if (button_pressed(1))
    {
			main_snake();
    }
}

#define F_CPU 			16000000
#define TRUE 			1
#define FALSE			0
#define PINITIAL_POS	16
#define BINITIAL_X		40
#define BINITIAL_Y		20
#define BOFFSET_Y		8
#define LEFT_COLUMN		1
#define RIGHT_COLUMN	80
#define TOP_LIMIT		1
#define BOTTOM_LIMIT	33
#define LEFT_PLIMIT		5
#define RIGHT_PLIMIT	76
#define SCORE_LIMIT		5

void calculate_diry(int dx, int dy, int *dirx, int *diry) {
	if (dy < -4)
		*diry = 2;
	else if (dy < -2)
		*diry = 1;
	else if (dy < 2)
		*diry = 0;
	else if (dy < 4)
		*diry = -1;
	else if (dy < 6)
		*diry = -2;
}

int main_pong(void)
{
	char left_platform_pos, right_platform_pos,
		 ball_x, ball_y;
	char pressed;
	int ball_dirx = 0, ball_diry = 0, dx = 0, dy = 0;
	char score_p1 = 0, score_p2 = 0;

	left_platform_pos = PINITIAL_POS;
	right_platform_pos = PINITIAL_POS;
	ball_x = BINITIAL_X;
	ball_y = BINITIAL_Y;

	/* Initializare LCD. */
	lcd_init();
	lcd_contrast(0x50);

	/* Ecran de start */
	lcd_goto_xy(6, 2);
	lcd_str("PONG");
	lcd_goto_xy(2, 3);
	lcd_str("Marius Avram");
	lcd_goto_xy(5, 5);
	lcd_str("334CB");

	DDRA = (1 << PA2);
	PORTA |= (1 << PA2);
	while (1) {
		if (!(PINA & (1 << PA2)))
		{
			lcd_clear();
			break;
		}
		_delay_ms(500);

	}

	/* Desenare minge */
	lcd_goto_xy_pixel(ball_x, ball_y);
	lcd_chr_unspaced(BALL);
	if (rand() % 2 == 0)
	{
		ball_dirx = 2;
	}
	else
	{
		ball_dirx = -2;
	}

	/* Activare pinii de input pentru butoane */
	DDRA = (1 << PA0) & (1 << PA1) &
			(1 << PA4) & (1 << PA5);
	/* Activare output pin led placuta imprimata */
	DDRD |= (1 << PD7);

	/* Citire input si afisare modificari pe ecran. */
	while (1)
	{
		lcd_clear();
		/* Desenare platforme */
		lcd_goto_xy_pixel(LEFT_COLUMN, left_platform_pos);
		draw_platform();
		lcd_goto_xy_pixel(RIGHT_COLUMN, right_platform_pos);
		draw_platform();

		pressed = FALSE;
		/* Platforma stanga */
		PORTA |= (1 << PA1);
		if (!(PINA & (1 << PA1)))
		{
			/* Buton jos */
			PORTD |= (1 << PD7);
			if (left_platform_pos < BOTTOM_LIMIT) {
				clear_left_column();
				lcd_goto_xy_pixel(LEFT_COLUMN, ++left_platform_pos);
				draw_platform();
				pressed = TRUE;
			}
		}
		PORTA |= (1 << PA0);
		if (!(PINA & (1 << PA0)))
		{
			/* Buton sus */
			PORTD |= (1 << PD7);
			if (left_platform_pos > TOP_LIMIT) {
				clear_left_column();
				lcd_goto_xy_pixel(LEFT_COLUMN, --left_platform_pos);
				draw_platform();
				pressed = TRUE;
			}
		}
		/* Platforma dreapta */
		PORTA |= (1 << PA4);
		if (!(PINA & (1 << PA4)))
		{
			/* Buton jos */
			PORTD |= (1 << PD7);
			if (right_platform_pos < BOTTOM_LIMIT) {
				clear_right_column();
				lcd_goto_xy_pixel(RIGHT_COLUMN, ++right_platform_pos);
				draw_platform();
				pressed = TRUE;
			}
		}
		PORTA |= (1 << PA5);
		if (!(PINA & (1 << PA5)))
		{
			/* Buton sus */
			PORTA |= (1 << PD7);
			if (right_platform_pos > TOP_LIMIT) {
				clear_right_column();
				lcd_goto_xy_pixel(RIGHT_COLUMN, --right_platform_pos);
				draw_platform();
				pressed = TRUE;
			}
		}

		if (!pressed)
		{
			PORTD &= ~(1 << PD7);
		}

		/* Redesenare minge */

		/* Ricosare partea stanga */
		if (ball_x == LEFT_PLIMIT+1 &&
			ball_y >= left_platform_pos-BOFFSET_Y &&
			ball_y <= left_platform_pos+12)
		{
			ball_dirx *= -1;
			dy = left_platform_pos - ball_y+3;
			dx = ball_dirx;
			calculate_diry(dx, dy, &ball_dirx, &ball_diry);
		}
		/* Ricosare partea dreapta */
		else if (ball_x == RIGHT_PLIMIT &&
			ball_y >= right_platform_pos-BOFFSET_Y &&
			ball_y <= right_platform_pos+12)
		{
			ball_dirx *= -1;
			dy = right_platform_pos - ball_y+3;
			dx = ball_dirx;
			calculate_diry(dx, dy, &ball_dirx, &ball_diry);
		}
		/* Mingea nu a fost prinsa */
		else if (ball_x <= LEFT_PLIMIT || ball_x >= RIGHT_PLIMIT)
		{
			/* Marcarea jucator 2 */
			if (ball_x <= LEFT_PLIMIT)
			{
				score_p2++;
			}
			/* Marcare jucator 1 */
			else if (ball_x >= RIGHT_PLIMIT)
			{
				score_p1++;
			}
			ball_dirx *= -1;
			ball_diry = 0;
			ball_x = BINITIAL_X;
			ball_y = BINITIAL_Y;
			/* Redeseneaza scena */
			lcd_clear();
			lcd_goto_xy_pixel(LEFT_COLUMN, left_platform_pos);
			draw_platform();
			lcd_goto_xy_pixel(RIGHT_COLUMN, right_platform_pos);
			draw_platform();
			lcd_goto_xy_pixel(ball_x, ball_y);
			lcd_chr_unspaced(BALL);
			if (score_p1 == SCORE_LIMIT)
			{
				lcd_goto_xy(3, 2);
				lcd_chr(ARROW_RIGHT);
				printf(" P1 WON! ");
				lcd_chr(ARROW_LEFT);
				break;
			}
			if (score_p2 == SCORE_LIMIT)
			{
				lcd_goto_xy(3, 2);
				lcd_chr(ARROW_RIGHT);
				printf(" P2 WON! ");
				lcd_chr(ARROW_LEFT);
				break;
			}
			_delay_ms(10000);
		}
		/* Lovire manta sus sau jos */
		if (ball_y < 1 || ball_y > 40) {
			ball_diry *= -1;
		}

		/* Progres minge */
		ball_x += ball_dirx;
		ball_y += ball_diry;

		lcd_goto_xy(6, 1);
		printf("%i - %i", score_p1, score_p2);
		lcd_goto_xy_pixel(ball_x, ball_y);
		lcd_chr_unspaced(BALL);

		_delay_ms(400);

	};
	/* Blocheaza jocul pana la reset. */
	while (1);
	return 0;
}

int main_snake(void)
{

	unsigned char ecran[46][12];
	unsigned char b0, b1, b2, b3, b4, freeze;
	unsigned int i, j, k, dir, c, p;

	//snake body coords on x and y
	unsigned char x[500], y[500];

	unsigned char dim_s = 8; // initial snake size is 8
	unsigned char x_tail, y_tail;

	b0 = b1 = b2 = b3 = b4 = 0;

	// initial snake position
	for (i = 0 ; i < dim_s; i++) {
		y[i] = 15;
		x[dim_s - 1 - i] = 15 - i;
	}

	// init ecran
	for (i = 0; i < 46; i ++) {
		for (j = 0; j < 10; j++) {
			ecran[i][j] = 0x00;
		}
	}

	// Setup LCD
	lcd_init();
	lcd_contrast(0x40);
	lcd_clear();


	// Print on first line
	lcd_goto_xy(1,1);
	lcd_str("Snake Game");

	// Print on second line with custom characters
	lcd_goto_xy(4,2);
	lcd_chr(ARROW_LEFT);
	printf(" PLAY ");
	lcd_chr(ARROW_RIGHT);

	DDRC = 0x00;
	PORTC = 0xFF;

	_delay_ms(10000);

	lcd_init();
	lcd_contrast(0x40);
	lcd_clear();

	freeze = 0;

	while(1) {

		// if pause button is pressed
		if (button_pressed(4)) {
			freeze = 1 - freeze;
			continue;
		}

		if (freeze) {
			continue;
		}

		lcd_reset_position();

		if (!button_pressed(0)) {
			b0 = 1;
		}
		if (b0 == 1 && button_pressed(0)) {
			if (dir!= 1) {
				dir = 3;
			}
		}

		if (!button_pressed(1)) {
			b1 = 1;
		}
		if (b1 == 1 && button_pressed(1)) {
			if (dir != 0) {
				dir = 2;
			}
		}

		if (!button_pressed(2)) {
			b2 = 1;
		}
		if (b2 == 1 && button_pressed(2)) {
			if (dir != 3) {
				dir = 1;
			}
		}

		if (!button_pressed(3)) {
			b3 = 1;
		}
		if (b3 == 1 && button_pressed(3)) {
			if (dir != 2) {
				dir = 0;
			}
		}

		// Setup pentru urmatoarea configuratie
		x_tail = x[0];
		y_tail = y[0];

		// sarpe un pas in fata
		for (i = 0 ; i < dim_s - 1 ; i++) {
			x[i] = x[i+1];
			y[i] = y[i+1];
		}

		//calcul urmatoare pozitie in functie de directie
		if (dir == 0) {
			x[dim_s - 1] = (x[dim_s - 1] + 1) % 84;
		}
		if (dir == 1) {
			y[dim_s - 1] = (y[dim_s - 1] + 1) % 48;
		}
		if (dir == 2) {
			x[dim_s - 1] = (x[dim_s - 1] - 1 == -1) ? 83 : x[dim_s - 1] - 1;
		}
		if (dir == 3) {
			y[dim_s - 1] = (y[dim_s - 1] - 1 == -1) ? 47 : y[dim_s - 1] - 1;
		}

		//creare cap nou
		ecran[y[dim_s - 1]][x[dim_s - 1] / 8] |= (1<<x[dim_s - 1] % 8);
		i = y[dim_s - 1]/8;
		i = i*8;
		j = x[dim_s - 1] / 8;
		c = 0x00;
		for(p = 0; p < 8; p++) {
			k = x[dim_s - 1] % 8;
			if(ecran[i+p][j] & (1<<k))
				c |= (1<<p);
		}

		lcd_reset_position();
		lcd_goto_xy_exact(x[dim_s-1]+1,i/8+1);
		lcd_print_part(c);

		// stergere coada
		ecran[y_tail][x_tail / 8] &= ~(1<<x_tail % 8);
		i = y_tail / 8;
		i = i * 8;
		j = x_tail / 8;
		c = 0;
		for(p = 0; p < 8; p++) {
			k = x_tail % 8;
			if(ecran[i+p][j] & (1<<k))
				c |= (1<<p);
		}

		lcd_reset_position();
		lcd_goto_xy_exact(x_tail + 1, i / 8 + 1);
		lcd_print_part(c);

		_delay_ms(1000);
	}
}
